/*
 * @COPYRIGHT@
 * 
 * x-kernel v3.2
 * 
 * Copyright (c) 1993,1991,1990  Arizona Board of Regents
 * 
 * @COPYRIGHT@
 *
 * $RCSfile: scout_fifo.c,v $
 *
 * HISTORY
 * $Log: scout_fifo.c,v $
 * Revision 1.3  1994/06/11  00:02:29  menze
 * Added copyright notice
 *
 * Revision 1.2  1994/05/14  23:54:00  davidm
 * (scout_fifo_append): It's now safe to call it concurrently.
 *
 * Revision 1.1  1994/05/05  00:27:41  davidm
 * Initial revision
 *
 * Revision 1.1  1994/01/26  01:51:14  davidm
 * Initial revision
 *
 */
/*
 * Concurrent single-reader/multi-writer FIFO queues.
 */
#include "scout_assert.h"
#include "scout_fifo.h"
#include "scout_sync_arch.h"

typedef struct scout_fifo_link Link;


void
scout_fifo_init(Scout_FIFO *q)
{
    q->head = q->tail = (void*) 0;
} /* scout_fifo_init */


void
scout_fifo_append(Scout_FIFO *q, void *element)
{
    Link *el = element;

    scout_assert(element);

    el->next = 0;
    ScoutAtomicSequence(if (q->head) {
			    q->tail->next = el;
			} else {
			    q->head = el;
			} /* if */
			q->tail = el);
} /* scout_fifo_append */


void*
scout_fifo_remove(Scout_FIFO *q)
{
    Link *el;

    if (!q->head) {
	/* cannot remove from empty fifo: */
	return 0;
    } /* if */

    el = q->head;
    if (el->next) {
	q->head = el->next;	/* more elements in the queue---no problem */
    } else {
	ScoutAtomicSequence(q->head = el->next);
    } /* if */
    return el;
} /* scout_fifo_remove */

			/*** end of scout_fifo.c ***/
